package org.bouncycastle.asn1;

import java.io.*;

/**
 * DER TaggedObject - in ASN.1 nottation this is any object proceeded by
 * a [n] where n is some number - these are assume to follow the construction
 * rules (as with sequences).
 */
public class DERTaggedObject
    extends DERObject
{
    int             tagNo;
    boolean         empty = false;
    DERObject       obj = null;

    /**
     * This creates an empty tagged object of tagNo (ie. zero length).
     *
     * @param tagNo the tag number for this object.
     */
    public DERTaggedObject(
        int     tagNo)
    {
        this.tagNo = tagNo;
        this.empty = true; 
    }

    /**
     * @param tagNo the tag number for this object.
     * @param obj the tagged object.
     */
    public DERTaggedObject(
        int         tagNo,
        DERObject   obj)
    {
        this.tagNo = tagNo;
        this.obj = obj;
    }

    public int getTagNo()
    {
        return tagNo;
    }

    public boolean isEmpty()
    {
        return empty;
    }

    public DERObject getObject()
    {
        return obj;
    }

    void encode(
        DEROutputStream  out)
        throws IOException
    {
        if (!empty)
        {
            ByteArrayOutputStream   bOut = new ByteArrayOutputStream();
            DEROutputStream         dOut = new DEROutputStream(bOut);

            dOut.writeObject(obj);
            dOut.close();

            out.writeEncoded(CONSTRUCTED | TAGGED | tagNo, bOut.toByteArray());
        }
        else
        {
            out.writeEncoded(CONSTRUCTED | TAGGED | tagNo, new byte[0]);
        }
    }
}
