package org.bouncycastle.asn1.pkcs;

import java.util.Enumeration;

import org.bouncycastle.asn1.*;

public class ContentInfo
    implements DEREncodable, PKCSObjectIdentifiers
{
    private DERObjectIdentifier contentType;
    private DERObject           content;

    public ContentInfo(
        DERConstructedSequence  obj)
    {
        Enumeration     e = obj.getObjects();

        contentType = (DERObjectIdentifier)e.nextElement();

        if (e.hasMoreElements())
        {
            content = ((DERTaggedObject)e.nextElement()).getObject();
        }
    }

    public DERObjectIdentifier getContentType()
    {
        return contentType;
    }

    public DERObject getContent()
    {
        return content;
    }

    /**
     * <pre>
     * ContentInfo ::= SEQUENCE {
     *          contentType ContentType,
     *          content
     *          [0] EXPLICIT ANY DEFINED BY contentType OPTIONAL }
     * </pre>
     */
    public DERObject getDERObject()
    {
        DERConstructedSequence  seq = new DERConstructedSequence();

        seq.addObject(contentType);

        if (content != null)
        {
            seq.addObject(new DERTaggedObject(0, content));
        }

        return seq;
    }
}
