package org.bouncycastle.asn1.test;

import java.math.BigInteger;
import java.io.*;

import org.bouncycastle.asn1.*;
import org.bouncycastle.asn1.pkcs.*;
import org.bouncycastle.asn1.x509.*;
import org.bouncycastle.util.encoders.Hex;

public class DERTest
    implements PKCSObjectIdentifiers
{
    public static void main(
        String[]    args)
        throws IOException
    {
        BigInteger  one = BigInteger.valueOf(1);
        BigInteger  two = BigInteger.valueOf(2);
        BigInteger  three = BigInteger.valueOf(3);
        BigInteger  four = BigInteger.valueOf(4);
        BigInteger  five = BigInteger.valueOf(5);
        BigInteger  six = BigInteger.valueOf(6);
        BigInteger  seven = BigInteger.valueOf(7);
        BigInteger  eight = BigInteger.valueOf(8);

        ByteArrayOutputStream   bOut = new ByteArrayOutputStream();
        DEROutputStream         dOut = new DEROutputStream(bOut);
        RSAPrivateKeyStructure  priv = new RSAPrivateKeyStructure(one, two, three, four, five, six, seven, eight);
        PrivateKeyInfo          info = new PrivateKeyInfo(new AlgorithmIdentifier(rsaEncryption), priv.getDERObject());

        dOut.writeObject(info);
        dOut.close();

        byte[]  data = bOut.toByteArray();

        System.out.println(new String(Hex.encode(data)));

        ByteArrayInputStream    bIn = new ByteArrayInputStream(data);
        DERInputStream          dIn = new DERInputStream(bIn);

        info = new PrivateKeyInfo((DERConstructedSequence)dIn.readObject());
        priv = new RSAPrivateKeyStructure((DERConstructedSequence)info.getPrivateKey());

        System.out.println(
                    priv.getModulus() + " "
                    + priv.getPublicExponent() + " " 
                    + priv.getPrivateExponent() + " " 
                    + priv.getPrime1() + " " 
                    + priv.getPrime2() + " " 
                    + priv.getExponent1() + " " 
                    + priv.getExponent2() + " " 
                    + priv.getCoefficient());

        //
        // X509 public key
        //
        bOut = new ByteArrayOutputStream();
        dOut = new DEROutputStream(bOut);
        SubjectPublicKeyInfo          pubInfo = new SubjectPublicKeyInfo(new AlgorithmIdentifier(rsaEncryption), new RSAPublicKeyStructure(one, two).getDERObject());

        dOut.writeObject(pubInfo);
        dOut.close();

        data = bOut.toByteArray();

        System.out.println(new String(Hex.encode(data)));

        bIn = new ByteArrayInputStream(data);
        dIn = new DERInputStream(bIn);

        pubInfo = new SubjectPublicKeyInfo((DERConstructedSequence)dIn.readObject());

        RSAPublicKeyStructure   pubKey = new RSAPublicKeyStructure((DERConstructedSequence)pubInfo.getPublicKey());

        System.out.println(
                    pubKey.getModulus() + " "
                    + pubKey.getPublicExponent());
    }
}
