import java.awt.*;
import java.awt.event.*;

import javax.microedition.midlet.*;


public class DynLayout extends MIDlet implements ActionListener, ItemListener {
    static final String FILLED = "Filled";
    static final String OUTLINE = "Outline";

    int width = 100;
    int height = 100;

    Frame frame = new Frame ("DynLayout");
    TextField widthField = new TextField ("100");
    TextField heightField = new TextField ("100");
    Choice cardChoice = new Choice ();
    Button applyButton = new Button ("Apply");
    DynPanel dynPanel = new DynPanel ();

    class RectComponent extends Component {

        boolean fill;

        public RectComponent (boolean fill) {
            this.fill = fill;
        }

        public void paint (Graphics g) {
            Dimension d = getSize ();

            g.setColor (Color.black);
            if (fill) 
                g.fillRect ((d.width - (width - 5)) / 2, 
                            (d.height - (height - 5)) / 2,
                            width - 5, height - 5);
            else
                g.drawRect ((d.width - (width - 5)) / 2, 
                            (d.height - (height - 5)) / 2,
                            width - 5, height - 5);
        }

        public Dimension getPreferredSize () {
            return new Dimension (width, height);
        }

        public Dimension getMinimumSize () {
            return new Dimension (width, height);
        }
    }
    
    RectComponent rect1 = new RectComponent (false);
    RectComponent rect2 = new RectComponent (true);


    class DynPanel extends Panel {

        public void doLayout () {

            removeAll ();
            Dimension d = getSize ();

            if (d.width >= width && d.height >= height) {
                
                boolean dw = d.width >= 2 * width;
                boolean dh = d.height >= 2 * height;

                if (dw | dh) {
                    if ((dw && !dh) || (dw && dh && d.width > d.height)) 
                        setLayout (new GridLayout (1, 2));
                    else 
                        setLayout (new GridLayout (2, 1));

                    add (rect1);
                    add (rect2);
                }
                else {
                    setLayout (new BorderLayout ());
                    add (cardChoice.getSelectedIndex () == 0 ?
                         rect1 : rect2);
                    cardChoice.setEnabled (true);
                }
            }
            else {
                setLayout (new BorderLayout ());
                ScrollPane scroll = new ScrollPane ();
                Panel panel = new Panel (new GridLayout (2, 1));
                panel.add (rect1);
                panel.add (rect2);
                scroll.add (panel);
                add (scroll);
            }

            super.doLayout ();
        }


        public Dimension getPreferredSize () {
            return new Dimension (2 * width, height);
        }
    }

    
    public DynLayout () {
        
        Panel labelPanel = new Panel (new GridLayout (0, 1));
        Panel fieldPanel = new Panel (new GridLayout (0, 1));
        Panel buttonPanel = new Panel (new GridLayout (0, 1));
        
        labelPanel.add (new Label ("Width: "));
        labelPanel.add (new Label ("Height: "));
        
        fieldPanel.add (widthField);
        fieldPanel.add (heightField);
        
        cardChoice.add (OUTLINE);
        cardChoice.add (FILLED);

        buttonPanel.add (cardChoice);
        buttonPanel.add (applyButton);

        applyButton.addActionListener (this);
        cardChoice.addItemListener (this);

        Panel controlPanel = new Panel (new BorderLayout ());
        controlPanel.add (labelPanel, BorderLayout.WEST);
        controlPanel.add (fieldPanel, BorderLayout.CENTER);
        controlPanel.add (buttonPanel, BorderLayout.EAST);
        
        frame.add (controlPanel, BorderLayout.NORTH);
        frame.add (dynPanel, BorderLayout.CENTER);

        frame.addWindowListener (new WindowAdapter () {
                public void windowClosing (WindowEvent ev) {
                    notifyDestroyed ();
                }
            });
                        
        frame.pack ();
    }

    
    public void itemStateChanged (ItemEvent ev) {

        if (ev.getStateChange () == ItemEvent.SELECTED) {
            
            dynPanel.invalidate ();
            frame.validate ();
            dynPanel.repaint ();
        }
    }    


    public void actionPerformed (ActionEvent ev) {

        width = Integer.parseInt (widthField.getText ());
        height = Integer.parseInt (heightField.getText ());

        rect1.invalidate ();
        rect2.invalidate ();
        frame.validate ();
    }
    

    public void startApp () {
        frame.show ();
    }

    public void pauseApp() {
    }    


    public void destroyApp (boolean unconditional) {
        frame.dispose ();
    }
}
