import java.awt.*;


public class TableLayout implements LayoutManager {

    int cols;
 
    public void addLayoutComponent (String where, Component component) {
    }
    

    public void removeLayoutComponent (Component component) {
    }


    public TableLayout (int cols) {
	this.cols = cols;
	
	if (cols < 1) 
	    throw new RuntimeException 
		("cols must be > 0");
    }
    
    
    public Dimension getMinimumSizes (Container container, 
				      int [] mcw, int [] mrh) {

	int count = container.getComponentCount ();

    	Dimension sum = new Dimension (0, 0);
	int i = 0;

	for (int y = 0; y < mrh.length; y++) {
	    for (int x = 0; x < cols && i < count; x ++) {
		Dimension ms = container.getComponent (i++).getMinimumSize ();
		mcw [x] = Math.max (mcw [x], ms.width);
		mrh [y] = Math.max (mrh [y], ms.height);
	    }
	    sum.height += mrh [y]; 
	}
	
	for (int x = 0; x < cols; x++)
	    sum.width += mcw [x];

	return sum;
    }

    int getRowCount (Container container) {
	return (container.getComponentCount () + cols - 1) / cols;
    }

    public Dimension minimumLayoutSize (Container container) {
	
	Insets insets = container.getInsets ();
	int rows = getRowCount (container);

	Dimension result = getMinimumSizes 
	    (container, new int [cols], new int [rows]);
	
	result.width += cols - 1 + insets.left + insets.right;
	result.height += rows - 1 + insets.top + insets.bottom;

	return result;
    }


    public Dimension preferredLayoutSize (Container container) {
	return minimumLayoutSize (container);
    }


    public void layoutContainer (Container container) {

	int count = container.getComponentCount ();
	int rows = getRowCount (container);

	if (count == 0) return;

	Insets insets = container.getInsets ();
	Dimension size = container.getSize ();
	
	int x0 = insets.left;
	int y0 = insets.top;
	
	int w0 = size.width - x0 - insets.right;
	int h0 = size.height - y0 - insets.bottom;

	int [] mcw = new int [cols];
	int [] mrh = new int [rows];

	Dimension min = getMinimumSizes (container, mcw, mrh);

	// calculate a scale factor

	int scx = ((w0-cols+1) << 8) / min.width;
	int scy = ((h0-rows+1) << 8) / min.height;
	int i = 0;

	for (int y = 0; y < rows; y++) {
	    int x1 = x0;
	    int h = (mrh [y] * scy) >> 8;
	    for (int x = 0; x < cols && i < count; x++) {
		int w = (mcw [x] * scx) >> 8;
		container.getComponent (i++).setBounds (x1, y0, w, h);
		x1 += w + 1;
	    }
	    y0 += h + 1;
	}
    }		
} 
