import java.io.*;
import java.net.*;
//import java.util.*;

public class ChatServer {

    int current = 0;
    String [] lines = new String [256]; 
    ServerSocket serverSocket;

    public ChatServer (int port) throws IOException {
	serverSocket = new ServerSocket (port);
	System.out.println ("Serving port: "+port);
    }


    public void run () {
	while (true) {
	    try {
		Socket socket = serverSocket.accept ();
		handleRequest  (new BufferedReader 
		    (new InputStreamReader (socket.getInputStream ())), 
			       new OutputStreamWriter (socket.getOutputStream ()));
		socket.close ();
	    }
	    catch (Exception e) {
		e.printStackTrace (System.err);
	    }
	}
    }


 
    public void handleRequest (BufferedReader reader, 
			       Writer writer) throws IOException {

	// analyze request line

	String request = reader.readLine ();
	System.out.println ("handling: "+request);
	int s0 = request.indexOf (' ');
	int s1 = request.indexOf (' ', s0+1);

	String method = request.substring (0, s0);
	String address = request.substring (s0+1, s1);
	
	// determine first line

	int start = current - 20;  // default;
	int cut = address.indexOf ("?start=");

	if (cut != -1) 
	    start = Integer.parseInt (address.substring (cut+7));

	
	// skip headers

	while (true) {
	    String s = reader.readLine ();
	    System.out.println ("header: "+s);
	    if (s == null || s.length () == 0) break;
	}

	// in case of post, store given string

	if (method.equalsIgnoreCase ("post")) {

	    String nick = reader.readLine ().substring (5);
	    String text = reader.readLine ().substring (5);

	    System.out.println ("nick="+nick);
	    System.out.println ("text="+text);
	    
	    lines [(current++) % lines.length] = nick + ": "+ text; 

	    // skip possible additional crlf from bad http implementations
	    // a safer mechanism would be to use the "content-length" header
	    if (reader.ready ()) reader.readLine ();
	}

	writer.write ("HTTP/1.0 200 OK\r\n");
	writer.write ("Content-Type: text/plain\r\n");
        writer.write ("Connnection: close\r\n");

	// Header is separated from content by a blank line.
	writer.write ("\r\n");

	writer.write (""+current+"\r\n");

	if (start < current - lines.length) start = current - lines.length;
	if (start < 0) start = 0;

	for (int i = start; i < current; i++) {
	    writer.write (lines [i % lines.length]);
	    writer.write ("\r\n");
	}
	    
	writer.flush ();
    }


    public static void main (String [] argv) throws IOException {

	if (argv.length == 0) 
	    new ChatServer (8080).run ();
	else if (argv.length == 1)
	    new ChatServer (Integer.parseInt (argv[0])).run ();
	else 
	    System.out.println ("Usage: java ChatServer [port]");

    }

}


